/*
Copyright (c) 2008-2010 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors
 may be used to endorse or promote products derived from this software
 without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Description: Contains the definition of the CLongNumber class. 
*/


/**
 @file
*/
#ifndef __LONGNUMBER_H__
#define __LONGNUMBER_H__

#include "digit.h"

/**
This class has an object of the TSglQue class, which represents the digits of an integer number list.
It is derived from the CActive class to handle asynchronous user inputs.
It has methods to:
- edit the linked list
- traverse the linked list using an iterator
*/
class CLongNumber : public CActive
	{
private:
	/**
	The enumeration that describes the operations performed by this class.
	*/
	enum TOption	{
					ETOptionNone,
					/**
					Call the TSglQue::AddFirst() function.
					*/
					EAddFirst, 
					/**
					Call the TSglQue::AddLast() function.
					*/
					EAddLast,
					/**
					Call the TSglQue::Remove() function.
					*/
					ERemove
					};
public:
	static CLongNumber* NewL(CConsoleBase* aConsole);
	void RunL();
	void DoCancel();
	void AddToScheduler();
	void ReadNumber();
	~CLongNumber();
	void PrintNumber();
	void InitializeIter();
	TBool GetNumber(TInt& aValue);
	TBool Slide();
private:
	CLongNumber(CConsoleBase* aConsole);
	void ReadFunc();
private:
	/**
	The singly linked list, which represents a number.
	*/
	TSglQue<TDigit> iNumber;
	/**
	The user console.
	*/
	CConsoleBase* iConsole;
	/** 
	The option selected by the user. Add an element to the start or end of the list.
	*/
	TOption iOption;
	/**
	The iterator of the singly linked list.
	*/
	TSglQueIter<TDigit> iIterator;
	};

#endif

