/*
Copyright (c) 2008-2010 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors
 may be used to endorse or promote products derived from this software
 without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Description: Contains definition of functions defined in the CAdder class. 
*/



/**
 @file
*/

#include "adder.h"

/**
Creates an object of the CAdder class.
@param aLongNumber A pointer to the object of the CLongNumber class.
@return A pointer to the object of the CAdder class.
*/
CAdder* CAdder::NewL(CLongNumber* aLongNumber)
	{
	CAdder* self = new (ELeave) CAdder(aLongNumber);
	return self;
	}

/**
Constructor.
*/
CAdder::CAdder(CLongNumber* aLongNumber):iLongNumber(aLongNumber)
	{
	}

/**
Performs the addition of all digits of the iLongNumber object.
@return The sum of all digits of the iLongNumber object.
*/
TInt CAdder::Add()
	{
	// Set the iterator to the start of the list.
	iLongNumber->InitializeIter();

	// Initialize sum to 0.
	TInt num;
	TInt sum = 0;

	// Iterate the list.
	// The iterator is pointing to a valid node.
	// Get the digit represented by this node.
	while(iLongNumber->GetNumber(num))
		{
		// Update the sum.
		sum += num;
		// Go to the next digit.
		iLongNumber->Slide();
		}

	return sum;
	}

/**
Destructor.
*/
CAdder::~CAdder()
	{
	}

